@@ -1,7 +1,5 @@
-# Author: David Goodger
-# Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 1.13 $
-# Date: $Date: 2003/01/01 15:50:23 $
+ # $Id: statemachine.py 7037 2011-05-19 08:56:27Z milde $
+# Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -110,8 +108,9 @@ __docformat__ = 'restructuredtext'
 
 import sys
 import re
-from types import SliceType as _SliceType
-
+import types
+import unicodedata
+from docutils.error_reporting import ErrorOutput
 
 class StateMachine:
 
@@ -170,6 +169,10 @@ class StateMachine:
         line changes.  Observers are called with one argument, ``self``.
         Cleared at the end of `run()`."""
 
+        self._stderr = ErrorOutput()
+        """Wrapper around sys.stderr catching en-/decoding errors"""
+
+
     def unlink(self):
         """Remove circular references to objects no longer required."""
         for state in self.states.values():
@@ -177,7 +180,7 @@ class StateMachine:
         self.states = None
 
     def run(self, input_lines, input_offset=0, context=None,
-            input_source=None):
+            input_source=None, initial_state=None):
         """
         Run the state machine on `input_lines`. Return results (a list).
 
@@ -197,6 +200,7 @@ class StateMachine:
           of the file.
         - `context`: application-specific storage.
         - `input_source`: name or path of source of `input_lines`.
+        - `initial_state`: name of initial state.
         """
         self.runtime_init()
         if isinstance(input_lines, StringList):
@@ -205,17 +209,17 @@ class StateMachine:
             self.input_lines = StringList(input_lines, source=input_source)
         self.input_offset = input_offset
         self.line_offset = -1
-        self.current_state = self.initial_state
+        self.current_state = initial_state or self.initial_state
         if self.debug:
-            print >>sys.stderr, (
-                '\nStateMachine.run: input_lines (line_offset=%s):\n| %s'
-                % (self.line_offset, '\n| '.join(self.input_lines)))
+            print >>self._stderr, (
+                u'\nStateMachine.run: input_lines (line_offset=%s):\n| %s'
+                % (self.line_offset, u'\n| '.join(self.input_lines)))
         transitions = None
         results = []
         state = self.get_state()
         try:
             if self.debug:
-                print >>sys.stderr, ('\nStateMachine.run: bof transition')
+                print >>self._stderr, '\nStateMachine.run: bof transition'
             context, result = state.bof(context)
             results.extend(result)
             while 1:
@@ -225,15 +229,15 @@ class StateMachine:
                         if self.debug:
                             source, offset = self.input_lines.info(
                                 self.line_offset)
-                            print >>sys.stderr, (
-                                '\nStateMachine.run: line (source=%r, '
-                                'offset=%r):\n| %s'
+                            print >>self._stderr, (
+                                u'\nStateMachine.run: line (source=%r, '
+                                u'offset=%r):\n| %s'
                                 % (source, offset, self.line))
                         context, next_state, result = self.check_line(
                             context, state, transitions)
                     except EOFError:
                         if self.debug:
-                            print >>sys.stderr, (
+                            print >>self._stderr, (
                                 '\nStateMachine.run: %s.eof transition'
                                 % state.__class__.__name__)
                         result = state.eof(context)
@@ -245,7 +249,7 @@ class StateMachine:
                     self.previous_line() # back up for another try
                     transitions = (exception.args[0],)
                     if self.debug:
-                        print >>sys.stderr, (
+                        print >>self._stderr, (
                               '\nStateMachine.run: TransitionCorrection to '
                               'state "%s", transition %s.'
                               % (state.__class__.__name__, transitions[0]))
@@ -258,7 +262,7 @@ class StateMachine:
                     else:
                         transitions = (exception.args[1],)
                     if self.debug:
-                        print >>sys.stderr, (
+                        print >>self._stderr, (
                               '\nStateMachine.run: StateCorrection to state '
                               '"%s", transition %s.'
                               % (next_state, transitions[0]))
@@ -266,7 +270,8 @@ class StateMachine:
                     transitions = None
                 state = self.get_state(next_state)
         except:
-            self.error()
+            if self.debug:
+                self.error()
             raise
         self.observers = []
         return results
@@ -281,11 +286,11 @@ class StateMachine:
         """
         if next_state:
             if self.debug and next_state != self.current_state:
-                print >>sys.stderr, \
-                      ('\nStateMachine.get_state: Changing state from '
-                       '"%s" to "%s" (input line %s).'
-                       % (self.current_state, next_state,
-                          self.abs_line_number()))
+                print >>self._stderr, (
+                    '\nStateMachine.get_state: Changing state from '
+                    '"%s" to "%s" (input line %s).'
+                    % (self.current_state, next_state,
+                       self.abs_line_number()))
             self.current_state = next_state
         try:
             return self.states[self.current_state]
@@ -343,6 +348,10 @@ class StateMachine:
         finally:
             self.notify_observers()
 
+    def get_source(self, line_offset):
+        """Return source of line at absolute line offset `line_offset`."""
+        return self.input_lines.source(line_offset - self.input_offset)
+
     def abs_line_offset(self):
         """Return line offset of current line, from beginning of file."""
         return self.line_offset + self.input_offset
@@ -351,11 +360,44 @@ class StateMachine:
         """Return line number of current line (counting from 1)."""
         return self.line_offset + self.input_offset + 1
 
+    def get_source_and_line(self, lineno=None):
+        """Return (source, line) tuple for current or given line number.
+
+        Looks up the source and line number in the `self.input_lines`
+        StringList instance to count for included source files.
+
+        If the optional argument `lineno` is given, convert it from an
+        absolute line number to the corresponding (source, line) pair.
+        """
+        if lineno is None:
+            offset = self.line_offset
+        else:
+            offset = lineno - self.input_offset - 1
+        try:
+            src, srcoffset = self.input_lines.info(offset)
+            srcline = srcoffset + 1
+        except (TypeError):
+            # line is None if index is "Just past the end"
+            src, srcline = self.get_source_and_line(offset + self.input_offset)
+            return src, srcline + 1
+        except (IndexError): # `offset` is off the list
+            src, srcline = None, None
+            # raise AssertionError('cannot find line %d in %s lines' %
+            #                      (offset, len(self.input_lines)))
+            #                      # list(self.input_lines.lines())))
+        # assert offset == srcoffset, str(self.input_lines)
+        # print "get_source_and_line(%s):" % lineno,
+        # print offset + 1, '->', src, srcline
+        # print self.input_lines
+        return (src, srcline)
+
     def insert_input(self, input_lines, source):
         self.input_lines.insert(self.line_offset + 1, '',
-                                source='internal padding')
+                                source='internal padding after '+source,
+                                offset=len(input_lines))
         self.input_lines.insert(self.line_offset + 1, '',
-                                source='internal padding')
+                                source='internal padding before '+source,
+                                offset=-1)
         self.input_lines.insert(self.line_offset + 2,
                                 StringList(input_lines, source))
 
@@ -373,7 +415,7 @@ class StateMachine:
             self.next_line(len(block) - 1)
             return block
         except UnexpectedIndentationError, error:
-            block, source, lineno = error
+            block, source, lineno = error.args
             self.next_line(len(block) - 1) # advance to last line of block
             raise
 
@@ -401,34 +443,26 @@ class StateMachine:
             transitions =  state.transition_order
         state_correction = None
         if self.debug:
-            print >>sys.stderr, (
+            print >>self._stderr, (
                   '\nStateMachine.check_line: state="%s", transitions=%r.'
                   % (state.__class__.__name__, transitions))
         for name in transitions:
             pattern, method, next_state = state.transitions[name]
-            match = self.match(pattern)
+            match = pattern.match(self.line)
             if match:
                 if self.debug:
-                    print >>sys.stderr, (
+                    print >>self._stderr, (
                           '\nStateMachine.check_line: Matched transition '
                           '"%s" in state "%s".'
                           % (name, state.__class__.__name__))
                 return method(match, context, next_state)
         else:
             if self.debug:
-                print >>sys.stderr, (
+                print >>self._stderr, (
                       '\nStateMachine.check_line: No match in state "%s".'
                       % state.__class__.__name__)
             return state.no_match(context, transitions)
 
-    def match(self, pattern):
-        """
-        Return the result of a regular expression match.
-
-        Parameter `pattern`: an `re` compiled regular expression.
-        """
-        return pattern.match(self.line)
-
     def add_state(self, state_class):
         """
         Initialize & add a `state_class` (`State` subclass) object.
@@ -437,7 +471,7 @@ class StateMachine:
         added.
         """
         statename = state_class.__name__
-        if self.states.has_key(statename):
+        if statename in self.states:
             raise DuplicateStateError(statename)
         self.states[statename] = state_class(self, self.debug)
 
@@ -458,10 +492,10 @@ class StateMachine:
     def error(self):
         """Report error details."""
         type, value, module, line, function = _exception_data()
-        print >>sys.stderr, '%s: %s' % (type, value)
-        print >>sys.stderr, 'input line %s' % (self.abs_line_number())
-        print >>sys.stderr, ('module %s, line %s, function %s'
-                             % (module, line, function))
+        print >>self._stderr, u'%s: %s' % (type, value)
+        print >>self._stderr, 'input line %s' % (self.abs_line_number())
+        print >>self._stderr, (u'module %s, line %s, function %s' %
+                               (module, line, function))
 
     def attach_observer(self, observer):
         """
@@ -625,9 +659,9 @@ class State:
         Exceptions: `DuplicateTransitionError`, `UnknownTransitionError`.
         """
         for name in names:
-            if self.transitions.has_key(name):
+            if name in self.transitions:
                 raise DuplicateTransitionError(name)
-            if not transitions.has_key(name):
+            if name not in transitions:
                 raise UnknownTransitionError(name)
         self.transition_order[:0] = names
         self.transitions.update(transitions)
@@ -640,7 +674,7 @@ class State:
 
         Exception: `DuplicateTransitionError`.
         """
-        if self.transitions.has_key(name):
+        if name in self.transitions:
             raise DuplicateTransitionError(name)
         self.transition_order[:0] = [name]
         self.transitions[name] = transition
@@ -760,7 +794,7 @@ class StateMachineWS(StateMachine):
     `StateMachine` subclass specialized for whitespace recognition.
 
     There are three methods provided for extracting indented text blocks:
-    
+
     - `get_indented()`: use when the indent is unknown.
     - `get_known_indented()`: use when the indent is known for all lines.
     - `get_first_known_indented()`: use when only the first line's indent is
@@ -1049,7 +1083,7 @@ class ViewList:
     child and parent lists can be broken by calling `disconnect()` on the
     child list.
 
-    Also, ViewList objects keep track of the source & offset of each item. 
+    Also, ViewList objects keep track of the source & offset of each item.
     This information is accessible via the `source()`, `offset()`, and
     `info()` methods.
     """
@@ -1106,21 +1140,20 @@ class ViewList:
     def __len__(self): return len(self.data)
 
     # The __getitem__()/__setitem__() methods check whether the index
-    # is a slice first, since native list objects start supporting
-    # them directly in Python 2.3 (no exception is raised when
-    # indexing a list with a slice object; they just work).
+    # is a slice first, since indexing a native list with a slice object
+    # just works.
 
     def __getitem__(self, i):
-        if isinstance(i, _SliceType):
+        if isinstance(i, types.SliceType):
             assert i.step in (None, 1),  'cannot handle slice with stride'
             return self.__class__(self.data[i.start:i.stop],
                                   items=self.items[i.start:i.stop],
-                                  parent=self, parent_offset=i.start)
+                                  parent=self, parent_offset=i.start or 0)
         else:
             return self.data[i]
 
     def __setitem__(self, i, item):
-        if isinstance(i, _SliceType):
+        if isinstance(i, types.SliceType):
             assert i.step in (None, 1), 'cannot handle slice with stride'
             if not isinstance(item, ViewList):
                 raise TypeError('assigning non-ViewList to ViewList slice')
@@ -1128,8 +1161,8 @@ class ViewList:
             self.items[i.start:i.stop] = item.items
             assert len(self.data) == len(self.items), 'data mismatch'
             if self.parent:
-                self.parent[i.start + self.parent_offset
-                            : i.stop + self.parent_offset] = item
+                self.parent[(i.start or 0) + self.parent_offset
+                            : (i.stop or len(self)) + self.parent_offset] = item
         else:
             self.data[i] = item
             if self.parent:
@@ -1146,8 +1179,8 @@ class ViewList:
             del self.data[i.start:i.stop]
             del self.items[i.start:i.stop]
             if self.parent:
-                del self.parent[i.start + self.parent_offset
-                                : i.stop + self.parent_offset]
+                del self.parent[(i.start or 0) + self.parent_offset
+                                : (i.stop or len(self)) + self.parent_offset]
 
     def __add__(self, other):
         if isinstance(other, ViewList):
@@ -1289,16 +1322,26 @@ class ViewList:
         """Break link between this list and parent list."""
         self.parent = None
 
+    def xitems(self):
+        """Return iterator yielding (source, offset, value) tuples."""
+        for (value, (source, offset)) in zip(self.data, self.items):
+            yield (source, offset, value)
+
+    def pprint(self):
+        """Print the list in `grep` format (`source:offset:value` lines)"""
+        for line in self.xitems():
+            print "%s:%d:%s" % line
+
 
 class StringList(ViewList):
 
     """A `ViewList` with string-specific methods."""
 
-    def strip_indent(self, length, start=0, end=sys.maxint):
+    def trim_left(self, length, start=0, end=sys.maxint):
         """
-        Strip `length` characters off the beginning of each item, in-place,
+        Trim `length` characters off the beginning of each item, in-place,
         from index `start` to `end`.  No whitespace-checking is done on the
-        stripped text.  Does not affect slice parent.
+        trimmed text.  Does not affect slice parent.
         """
         self.data[start:end] = [line[length:]
                                 for line in self.data[start:end]]
@@ -1381,9 +1424,44 @@ class StringList(ViewList):
         if first_indent is not None and block:
             block.data[0] = block.data[0][first_indent:]
         if indent and strip_indent:
-            block.strip_indent(indent, start=(first_indent is not None))
+            block.trim_left(indent, start=(first_indent is not None))
         return block, indent or 0, blank_finish
 
+    def get_2D_block(self, top, left, bottom, right, strip_indent=1):
+        block = self[top:bottom]
+        indent = right
+        for i in range(len(block.data)):
+            block.data[i] = line = block.data[i][left:right].rstrip()
+            if line:
+                indent = min(indent, len(line) - len(line.lstrip()))
+        if strip_indent and 0 < indent < right:
+            block.data = [line[indent:] for line in block.data]
+        return block
+
+    def pad_double_width(self, pad_char):
+        """
+        Pad all double-width characters in self by appending `pad_char` to each.
+        For East Asian language support.
+        """
+        if hasattr(unicodedata, 'east_asian_width'):
+            east_asian_width = unicodedata.east_asian_width
+        else:
+            return                      # new in Python 2.4
+        for i in range(len(self.data)):
+            line = self.data[i]
+            if isinstance(line, unicode):
+                new = []
+                for char in line:
+                    new.append(char)
+                    if east_asian_width(char) in 'WF': # 'W'ide & 'F'ull-width
+                        new.append(pad_char)
+                self.data[i] = ''.join(new)
+
+    def replace(self, old, new):
+        """Replace all occurrences of substring `old` with `new`."""
+        for i in range(len(self.data)):
+            self.data[i] = self.data[i].replace(old, new)
+
 
 class StateMachineError(Exception): pass
 class UnknownStateError(StateMachineError): pass
@@ -1417,7 +1495,8 @@ class StateCorrection(Exception):
 def string2lines(astring, tab_width=8, convert_whitespace=0,
                  whitespace=re.compile('[\v\f]')):
     """
-    Return a list of one-line strings with tabs expanded and no newlines.
+    Return a list of one-line strings with tabs expanded, no newlines, and
+    trailing whitespace stripped.
 
     Each tab is expanded with between 1 and `tab_width` spaces, so that the
     next character's index becomes a multiple of `tab_width` (8 by default).
@@ -1430,7 +1509,7 @@ def string2lines(astring, tab_width=8, convert_whitespace=0,
     """
     if convert_whitespace:
         astring = whitespace.sub(' ', astring)
-    return [s.expandtabs(tab_width) for s in astring.splitlines()]
+    return [s.expandtabs(tab_width).rstrip() for s in astring.splitlines()]
 
 def _exception_data():
     """
